"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.awsApiRequestSOAPAllItems = exports.awsApiRequestSOAP = exports.awsApiRequestREST = exports.awsApiRequest = void 0;
const lodash_1 = require("lodash");
const xml2js_1 = require("xml2js");
async function awsApiRequest(service, method, path, body, query = {}, headers, option = {}, _region) {
    const requestOptions = {
        qs: {
            ...query,
            service,
            path,
            query,
        },
        method,
        body,
        url: '',
        headers,
    };
    if (Object.keys(option).length !== 0) {
        Object.assign(requestOptions, option);
    }
    return this.helpers.requestWithAuthentication.call(this, 'aws', requestOptions);
}
exports.awsApiRequest = awsApiRequest;
async function awsApiRequestREST(service, method, path, body, query = {}, headers, options = {}, region) {
    const response = await awsApiRequest.call(this, service, method, path, body, query, headers, options, region);
    try {
        return JSON.parse(response);
    }
    catch (error) {
        return response;
    }
}
exports.awsApiRequestREST = awsApiRequestREST;
async function awsApiRequestSOAP(service, method, path, body, query = {}, headers, option = {}, region) {
    const response = await awsApiRequest.call(this, service, method, path, body, query, headers, option, region);
    try {
        return await new Promise((resolve, reject) => {
            (0, xml2js_1.parseString)(response, { explicitArray: false }, (err, data) => {
                if (err) {
                    return reject(err);
                }
                resolve(data);
            });
        });
    }
    catch (error) {
        return error;
    }
}
exports.awsApiRequestSOAP = awsApiRequestSOAP;
async function awsApiRequestSOAPAllItems(propertyName, service, method, path, body, query = {}, headers = {}, option = {}, region) {
    const returnData = [];
    let responseData;
    do {
        responseData = await awsApiRequestSOAP.call(this, service, method, path, body, query, headers, option, region);
        if ((0, lodash_1.get)(responseData, `${propertyName.split('.')[0]}.NextContinuationToken`)) {
            query['continuation-token'] = (0, lodash_1.get)(responseData, `${propertyName.split('.')[0]}.NextContinuationToken`);
        }
        if ((0, lodash_1.get)(responseData, propertyName)) {
            if (Array.isArray((0, lodash_1.get)(responseData, propertyName))) {
                returnData.push.apply(returnData, (0, lodash_1.get)(responseData, propertyName));
            }
            else {
                returnData.push((0, lodash_1.get)(responseData, propertyName));
            }
        }
        if (query.limit && query.limit <= returnData.length) {
            return returnData;
        }
    } while ((0, lodash_1.get)(responseData, `${propertyName.split('.')[0]}.IsTruncated`) !== undefined &&
        (0, lodash_1.get)(responseData, `${propertyName.split('.')[0]}.IsTruncated`) !== 'false');
    return returnData;
}
exports.awsApiRequestSOAPAllItems = awsApiRequestSOAPAllItems;
//# sourceMappingURL=GenericFunctions.js.map