"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkInput = exports.checkMatchFieldsInput = exports.findMatches = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const lodash_1 = require("lodash");
function compareItems(item1, item2, fieldsToMatch, resolve, skipFields, isEntriesEqual) {
    const keys = {};
    fieldsToMatch.forEach((field) => {
        keys[field.field1] = item1.json[field.field1];
    });
    const keys1 = Object.keys(item1.json);
    const keys2 = Object.keys(item2.json);
    const intersectionKeys = (0, lodash_1.intersection)(keys1, keys2);
    const same = intersectionKeys.reduce((acc, key) => {
        if (isEntriesEqual(item1.json[key], item2.json[key])) {
            acc[key] = item1.json[key];
        }
        return acc;
    }, {});
    const sameKeys = Object.keys(same);
    const allUniqueKeys = (0, lodash_1.union)(keys1, keys2);
    const differentKeys = (0, lodash_1.difference)(allUniqueKeys, sameKeys);
    const different = {};
    const skipped = {};
    differentKeys.forEach((key) => {
        switch (resolve) {
            case 'preferInput1':
                different[key] = item1.json[key] || null;
                break;
            case 'preferInput2':
                different[key] = item2.json[key] || null;
                break;
            default:
                const input1 = item1.json[key] || null;
                const input2 = item2.json[key] || null;
                if (skipFields.includes(key)) {
                    skipped[key] = { input1, input2 };
                }
                else {
                    different[key] = { input1, input2 };
                }
        }
    });
    return {
        json: { keys, same, different, ...(!(0, lodash_1.isEmpty)(skipped) && { skipped }) },
    };
}
function combineItems(item1, item2, prefer, except, disableDotNotation) {
    let exceptFields;
    const [entry, match] = prefer === 'input1' ? [item1, item2] : [item2, item1];
    if (except && Array.isArray(except) && except.length) {
        exceptFields = except;
    }
    else {
        exceptFields = except ? except.split(',').map((field) => field.trim()) : [];
    }
    exceptFields.forEach((field) => {
        entry.json[field] = match.json[field];
        if (disableDotNotation) {
            entry.json[field] = match.json[field];
        }
        else {
            const value = (0, lodash_1.get)(match.json, field) || null;
            (0, lodash_1.set)(entry, `json.${field}`, value);
        }
    });
    return entry;
}
function findAllMatches(data, lookup, disableDotNotation, isEntriesEqual) {
    return data.reduce((acc, entry2, i) => {
        if (entry2 === undefined)
            return acc;
        for (const key of Object.keys(lookup)) {
            const excpectedValue = lookup[key];
            let entry2FieldValue;
            if (disableDotNotation) {
                entry2FieldValue = entry2.json[key];
            }
            else {
                entry2FieldValue = (0, lodash_1.get)(entry2.json, key);
            }
            if (!isEntriesEqual(excpectedValue, entry2FieldValue)) {
                return acc;
            }
        }
        return acc.concat({
            entry: entry2,
            index: i,
        });
    }, []);
}
function findFirstMatch(data, lookup, disableDotNotation, isEntriesEqual) {
    const index = data.findIndex((entry2) => {
        if (entry2 === undefined)
            return false;
        for (const key of Object.keys(lookup)) {
            const excpectedValue = lookup[key];
            let entry2FieldValue;
            if (disableDotNotation) {
                entry2FieldValue = entry2.json[key];
            }
            else {
                entry2FieldValue = (0, lodash_1.get)(entry2.json, key);
            }
            if (!isEntriesEqual(excpectedValue, entry2FieldValue)) {
                return false;
            }
        }
        return true;
    });
    if (index === -1)
        return [];
    return [{ entry: data[index], index }];
}
const parseStringAndCompareToObject = (str, arr) => {
    try {
        const parsedArray = (0, n8n_workflow_1.jsonParse)(str);
        return (0, lodash_1.isEqual)(parsedArray, arr);
    }
    catch (error) {
        return false;
    }
};
function isFalsy(value) {
    if ((0, lodash_1.isNull)(value))
        return true;
    if (typeof value === 'string' && value === '')
        return true;
    if (Array.isArray(value) && value.length === 0)
        return true;
    return false;
}
const fuzzyCompare = (options) => (item1, item2) => {
    if (!options.fuzzyCompare)
        return (0, lodash_1.isEqual)(item1, item2);
    if (!(0, lodash_1.isNull)(item1) && !(0, lodash_1.isNull)(item2) && typeof item1 === typeof item2) {
        return (0, lodash_1.isEqual)(item1, item2);
    }
    if (isFalsy(item1) && isFalsy(item2))
        return true;
    if (isFalsy(item1) && item2 === undefined)
        return true;
    if (item1 === undefined && isFalsy(item2))
        return true;
    if (typeof item1 === 'number' && typeof item2 === 'string') {
        return item1.toString() === item2;
    }
    if (typeof item1 === 'string' && typeof item2 === 'number') {
        return item1 === item2.toString();
    }
    if (!(0, lodash_1.isNull)(item1) && typeof item1 === 'object' && typeof item2 === 'string') {
        return parseStringAndCompareToObject(item2, item1);
    }
    if (!(0, lodash_1.isNull)(item2) && typeof item1 === 'string' && typeof item2 === 'object') {
        return parseStringAndCompareToObject(item1, item2);
    }
    if (typeof item1 === 'boolean' && typeof item2 === 'string') {
        if (item1 === true && item2.toLocaleLowerCase() === 'true')
            return true;
        if (item1 === false && item2.toLocaleLowerCase() === 'false')
            return true;
    }
    if (typeof item2 === 'boolean' && typeof item1 === 'string') {
        if (item2 === true && item1.toLocaleLowerCase() === 'true')
            return true;
        if (item2 === false && item1.toLocaleLowerCase() === 'false')
            return true;
    }
    if (typeof item1 === 'boolean' && typeof item2 === 'number') {
        if (item1 === true && item2 === 1)
            return true;
        if (item1 === false && item2 === 0)
            return true;
    }
    if (typeof item2 === 'boolean' && typeof item1 === 'number') {
        if (item2 === true && item1 === 1)
            return true;
        if (item2 === false && item1 === 0)
            return true;
    }
    if (typeof item1 === 'boolean' && typeof item2 === 'string') {
        if (item1 === true && item2 === '1')
            return true;
        if (item1 === false && item2 === '0')
            return true;
    }
    if (typeof item2 === 'boolean' && typeof item1 === 'string') {
        if (item2 === true && item1 === '1')
            return true;
        if (item2 === false && item1 === '0')
            return true;
    }
    return (0, lodash_1.isEqual)(item1, item2);
};
function findMatches(input1, input2, fieldsToMatch, options) {
    const data1 = [...input1];
    const data2 = [...input2];
    const isEntriesEqual = fuzzyCompare(options);
    const disableDotNotation = options.disableDotNotation || false;
    const multipleMatches = options.multipleMatches || 'first';
    const skipFields = (options.skipFields || '').split(',').map((field) => field.trim());
    const filteredData = {
        matched: [],
        unmatched1: [],
        unmatched2: [],
    };
    const matchedInInput2 = new Set();
    matchesLoop: for (const entry of data1) {
        const lookup = {};
        fieldsToMatch.forEach((matchCase) => {
            let valueToCompare;
            if (disableDotNotation) {
                valueToCompare = entry.json[matchCase.field1];
            }
            else {
                valueToCompare = (0, lodash_1.get)(entry.json, matchCase.field1);
            }
            lookup[matchCase.field2] = valueToCompare;
        });
        for (const fieldValue of Object.values(lookup)) {
            if (fieldValue === undefined) {
                filteredData.unmatched1.push(entry);
                continue matchesLoop;
            }
        }
        const foundedMatches = multipleMatches === 'all'
            ? findAllMatches(data2, lookup, disableDotNotation, isEntriesEqual)
            : findFirstMatch(data2, lookup, disableDotNotation, isEntriesEqual);
        const matches = foundedMatches.map((match) => match.entry);
        foundedMatches.map((match) => matchedInInput2.add(match.index));
        if (matches.length) {
            filteredData.matched.push({ entry, matches });
        }
        else {
            filteredData.unmatched1.push(entry);
        }
    }
    data2.forEach((entry, i) => {
        if (!matchedInInput2.has(i)) {
            filteredData.unmatched2.push(entry);
        }
    });
    const same = [];
    const different = [];
    filteredData.matched.forEach((entryMatches) => {
        let entryCopy;
        entryMatches.matches.forEach((match) => {
            let entryFromInput1 = entryMatches.entry.json;
            let entryFromInput2 = match.json;
            if (skipFields.length) {
                entryFromInput1 = (0, lodash_1.omit)(entryFromInput1, skipFields);
                entryFromInput2 = (0, lodash_1.omit)(entryFromInput2, skipFields);
            }
            let isItemsEqual = true;
            if (options.fuzzyCompare) {
                for (const key of Object.keys(entryFromInput1)) {
                    if (!isEntriesEqual(entryFromInput1[key], entryFromInput2[key])) {
                        isItemsEqual = false;
                        break;
                    }
                }
            }
            else {
                isItemsEqual = isEntriesEqual(entryFromInput1, entryFromInput2);
            }
            if (isItemsEqual) {
                if (!entryCopy) {
                    if (options.fuzzyCompare && options.resolve === 'preferInput2') {
                        entryCopy = match;
                    }
                    else {
                        entryCopy = entryMatches.entry;
                    }
                }
            }
            else {
                switch (options.resolve) {
                    case 'preferInput1':
                        different.push(entryMatches.entry);
                        break;
                    case 'preferInput2':
                        different.push(match);
                        break;
                    case 'mix':
                        different.push(combineItems(entryMatches.entry, match, options.preferWhenMix, options.exceptWhenMix, disableDotNotation));
                        break;
                    default:
                        different.push(compareItems(entryMatches.entry, match, fieldsToMatch, options.resolve, skipFields, isEntriesEqual));
                }
            }
        });
        if (!(0, lodash_1.isEmpty)(entryCopy)) {
            same.push(entryCopy);
        }
    });
    return [filteredData.unmatched1, same, different, filteredData.unmatched2];
}
exports.findMatches = findMatches;
function checkMatchFieldsInput(data) {
    if (data.length === 1 && data[0].field1 === '' && data[0].field2 === '') {
        throw new Error('You need to define at least one pair of fields in "Fields to Match" to match on');
    }
    for (const [index, pair] of data.entries()) {
        if (pair.field1 === '' || pair.field2 === '') {
            throw new Error(`You need to define both fields in "Fields to Match" for pair ${index + 1},
				 field 1 = '${pair.field1}'
				 field 2 = '${pair.field2}'`);
        }
    }
    return data;
}
exports.checkMatchFieldsInput = checkMatchFieldsInput;
function checkInput(input, fields, disableDotNotation, inputLabel) {
    if (input.some((item) => (0, lodash_1.isEmpty)(item.json))) {
        input = input.filter((item) => !(0, lodash_1.isEmpty)(item.json));
    }
    if (input.length === 0) {
        return input;
    }
    for (const field of fields) {
        const isPresent = (input || []).some((entry) => {
            if (disableDotNotation) {
                return entry.json.hasOwnProperty(field);
            }
            return (0, lodash_1.get)(entry.json, field, undefined) !== undefined;
        });
        if (!isPresent) {
            throw new Error(`Field '${field}' is not present in any of items in '${inputLabel}'`);
        }
    }
    return input;
}
exports.checkInput = checkInput;
//# sourceMappingURL=GenericFunctions.js.map