"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Helpers_1 = require("../../../../test/nodes/Helpers");
const nock_1 = __importDefault(require("nock"));
describe('Test HTTP Request Node', () => {
    const workflows = (0, Helpers_1.getWorkflowFilenames)(__dirname);
    const tests = (0, Helpers_1.workflowToTests)(workflows);
    const baseUrl = 'https://dummyjson.com';
    beforeAll(() => {
        nock_1.default.disableNetConnect();
        (0, nock_1.default)(baseUrl).get('/todos/1').reply(200, {
            id: 1,
            todo: 'Do something nice for someone I care about',
            completed: true,
            userId: 26,
        });
        (0, nock_1.default)(baseUrl).matchHeader('Authorization', 'Bearer 12345').get('/todos/3').reply(200, {
            id: 3,
            todo: 'Watch a classic movie',
            completed: false,
            userId: 4,
        });
        (0, nock_1.default)(baseUrl)
            .get('/todos?limit=2&skip=10')
            .reply(200, {
            todos: [
                {
                    id: 11,
                    todo: "Text a friend I haven't talked to in a long time",
                    completed: false,
                    userId: 39,
                },
                {
                    id: 12,
                    todo: 'Organize pantry',
                    completed: true,
                    userId: 39,
                },
            ],
            total: 150,
            skip: 10,
            limit: 2,
        });
        (0, nock_1.default)(baseUrl)
            .post('/todos/add', {
            todo: 'Use DummyJSON in the project',
            completed: false,
            userId: '5',
        })
            .reply(200, {
            id: 151,
            todo: 'Use DummyJSON in the project',
            completed: false,
            userId: '5',
        });
        (0, nock_1.default)(baseUrl)
            .post('/todos/add2', {
            todo: 'Use DummyJSON in the project',
            completed: false,
            userId: 15,
        })
            .reply(200, {
            id: 151,
            todo: 'Use DummyJSON in the project',
            completed: false,
            userId: 15,
        });
        (0, nock_1.default)(baseUrl).put('/todos/10', { userId: '42' }).reply(200, {
            id: 10,
            todo: 'Have a football scrimmage with some friends',
            completed: false,
            userId: '42',
        });
        (0, nock_1.default)(baseUrl)
            .patch('/products/1', '{"title":"iPhone 12"}')
            .reply(200, {
            id: 1,
            title: 'iPhone 12',
            price: 549,
            stock: 94,
            rating: 4.69,
            images: [
                'https://i.dummyjson.com/data/products/1/1.jpg',
                'https://i.dummyjson.com/data/products/1/2.jpg',
                'https://i.dummyjson.com/data/products/1/3.jpg',
                'https://i.dummyjson.com/data/products/1/4.jpg',
                'https://i.dummyjson.com/data/products/1/thumbnail.jpg',
            ],
            thumbnail: 'https://i.dummyjson.com/data/products/1/thumbnail.jpg',
            description: 'An apple mobile which is nothing like apple',
            brand: 'Apple',
            category: 'smartphones',
        });
        (0, nock_1.default)(baseUrl).delete('/todos/1').reply(200, {
            id: 1,
            todo: 'Do something nice for someone I care about',
            completed: true,
            userId: 26,
            isDeleted: true,
            deletedOn: '2023-02-09T05:37:31.720Z',
        });
    });
    afterAll(() => {
        nock_1.default.restore();
    });
    const nodeTypes = (0, Helpers_1.setup)(tests);
    for (const testData of tests) {
        test(testData.description, async () => (0, Helpers_1.equalityTest)(testData, nodeTypes));
    }
});
//# sourceMappingURL=HttpRequest.test.js.map