"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addSourceField = exports.checkInput = exports.checkMatchFieldsInput = exports.mergeMatched = exports.selectMergeMethod = exports.findMatches = exports.addSuffixToEntriesKeys = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const lodash_1 = require("lodash");
function addSuffixToEntriesKeys(data, suffix) {
    return data.map((entry) => {
        const json = {};
        Object.keys(entry.json).forEach((key) => {
            json[`${key}_${suffix}`] = entry.json[key];
        });
        return { ...entry, json };
    });
}
exports.addSuffixToEntriesKeys = addSuffixToEntriesKeys;
function findAllMatches(data, lookup, disableDotNotation, isEntriesEqual) {
    return data.reduce((acc, entry2, i) => {
        if (entry2 === undefined)
            return acc;
        for (const key of Object.keys(lookup)) {
            const excpectedValue = lookup[key];
            let entry2FieldValue;
            if (disableDotNotation) {
                entry2FieldValue = entry2.json[key];
            }
            else {
                entry2FieldValue = (0, lodash_1.get)(entry2.json, key);
            }
            if (!isEntriesEqual(excpectedValue, entry2FieldValue)) {
                return acc;
            }
        }
        return acc.concat({
            entry: entry2,
            index: i,
        });
    }, []);
}
function findFirstMatch(data, lookup, disableDotNotation, isEntriesEqual) {
    const index = data.findIndex((entry2) => {
        if (entry2 === undefined)
            return false;
        for (const key of Object.keys(lookup)) {
            const excpectedValue = lookup[key];
            let entry2FieldValue;
            if (disableDotNotation) {
                entry2FieldValue = entry2.json[key];
            }
            else {
                entry2FieldValue = (0, lodash_1.get)(entry2.json, key);
            }
            if (!isEntriesEqual(excpectedValue, entry2FieldValue)) {
                return false;
            }
        }
        return true;
    });
    if (index === -1)
        return [];
    return [{ entry: data[index], index }];
}
const parseStringAndCompareToObject = (str, arr) => {
    try {
        const parsedArray = (0, n8n_workflow_1.jsonParse)(str);
        return (0, lodash_1.isEqual)(parsedArray, arr);
    }
    catch (error) {
        return false;
    }
};
function isFalsy(value) {
    if ((0, lodash_1.isNull)(value))
        return true;
    if (typeof value === 'string' && value === '')
        return true;
    if (Array.isArray(value) && value.length === 0)
        return true;
    return false;
}
const fuzzyCompare = (options) => (item1, item2) => {
    if (!options.fuzzyCompare)
        return (0, lodash_1.isEqual)(item1, item2);
    if (!(0, lodash_1.isNull)(item1) && !(0, lodash_1.isNull)(item2) && typeof item1 === typeof item2) {
        return (0, lodash_1.isEqual)(item1, item2);
    }
    if (isFalsy(item1) && isFalsy(item2))
        return true;
    if (isFalsy(item1) && item2 === undefined)
        return true;
    if (item1 === undefined && isFalsy(item2))
        return true;
    if (typeof item1 === 'number' && typeof item2 === 'string') {
        return item1.toString() === item2;
    }
    if (typeof item1 === 'string' && typeof item2 === 'number') {
        return item1 === item2.toString();
    }
    if (!(0, lodash_1.isNull)(item1) && typeof item1 === 'object' && typeof item2 === 'string') {
        return parseStringAndCompareToObject(item2, item1);
    }
    if (!(0, lodash_1.isNull)(item2) && typeof item1 === 'string' && typeof item2 === 'object') {
        return parseStringAndCompareToObject(item1, item2);
    }
    if (typeof item1 === 'boolean' && typeof item2 === 'string') {
        if (item1 === true && item2.toLocaleLowerCase() === 'true')
            return true;
        if (item1 === false && item2.toLocaleLowerCase() === 'false')
            return true;
    }
    if (typeof item2 === 'boolean' && typeof item1 === 'string') {
        if (item2 === true && item1.toLocaleLowerCase() === 'true')
            return true;
        if (item2 === false && item1.toLocaleLowerCase() === 'false')
            return true;
    }
    if (typeof item1 === 'boolean' && typeof item2 === 'number') {
        if (item1 === true && item2 === 1)
            return true;
        if (item1 === false && item2 === 0)
            return true;
    }
    if (typeof item2 === 'boolean' && typeof item1 === 'number') {
        if (item2 === true && item1 === 1)
            return true;
        if (item2 === false && item1 === 0)
            return true;
    }
    if (typeof item1 === 'boolean' && typeof item2 === 'string') {
        if (item1 === true && item2 === '1')
            return true;
        if (item1 === false && item2 === '0')
            return true;
    }
    if (typeof item2 === 'boolean' && typeof item1 === 'string') {
        if (item2 === true && item1 === '1')
            return true;
        if (item2 === false && item1 === '0')
            return true;
    }
    return (0, lodash_1.isEqual)(item1, item2);
};
function findMatches(input1, input2, fieldsToMatch, options) {
    let data1 = [...input1];
    let data2 = [...input2];
    if (options.joinMode === 'enrichInput2') {
        [data1, data2] = [data2, data1];
    }
    const isEntriesEqual = fuzzyCompare(options);
    const disableDotNotation = options.disableDotNotation || false;
    const multipleMatches = options.multipleMatches || 'all';
    const filteredData = {
        matched: [],
        matched2: [],
        unmatched1: [],
        unmatched2: [],
    };
    const matchedInInput2 = new Set();
    matchesLoop: for (const entry1 of data1) {
        const lookup = {};
        fieldsToMatch.forEach((matchCase) => {
            let valueToCompare;
            if (disableDotNotation) {
                valueToCompare = entry1.json[matchCase.field1];
            }
            else {
                valueToCompare = (0, lodash_1.get)(entry1.json, matchCase.field1);
            }
            lookup[matchCase.field2] = valueToCompare;
        });
        for (const fieldValue of Object.values(lookup)) {
            if (fieldValue === undefined) {
                filteredData.unmatched1.push(entry1);
                continue matchesLoop;
            }
        }
        const foundedMatches = multipleMatches === 'all'
            ? findAllMatches(data2, lookup, disableDotNotation, isEntriesEqual)
            : findFirstMatch(data2, lookup, disableDotNotation, isEntriesEqual);
        const matches = foundedMatches.map((match) => match.entry);
        foundedMatches.map((match) => matchedInInput2.add(match.index));
        if (matches.length) {
            if (options.outputDataFrom === 'both' ||
                options.joinMode === 'enrichInput1' ||
                options.joinMode === 'enrichInput2') {
                matches.forEach((match) => {
                    filteredData.matched.push({
                        entry: entry1,
                        matches: [match],
                    });
                });
            }
            else {
                filteredData.matched.push({
                    entry: entry1,
                    matches,
                });
            }
        }
        else {
            filteredData.unmatched1.push(entry1);
        }
    }
    data2.forEach((entry, i) => {
        if (matchedInInput2.has(i)) {
            filteredData.matched2.push(entry);
        }
        else {
            filteredData.unmatched2.push(entry);
        }
    });
    return filteredData;
}
exports.findMatches = findMatches;
function selectMergeMethod(clashResolveOptions) {
    const mergeMode = clashResolveOptions.mergeMode;
    if (clashResolveOptions.overrideEmpty) {
        function customizer(targetValue, srcValue) {
            if (srcValue === undefined || srcValue === null || srcValue === '') {
                return targetValue;
            }
        }
        if (mergeMode === 'deepMerge') {
            return (target, ...source) => {
                const targetCopy = Object.assign({}, target);
                return (0, lodash_1.mergeWith)(targetCopy, ...source, customizer);
            };
        }
        if (mergeMode === 'shallowMerge') {
            return (target, ...source) => {
                const targetCopy = Object.assign({}, target);
                return (0, lodash_1.assignWith)(targetCopy, ...source, customizer);
            };
        }
    }
    else {
        if (mergeMode === 'deepMerge') {
            return (target, ...source) => (0, lodash_1.merge)({}, target, ...source);
        }
        if (mergeMode === 'shallowMerge') {
            return (target, ...source) => (0, lodash_1.assign)({}, target, ...source);
        }
    }
    return (target, ...source) => (0, lodash_1.merge)({}, target, ...source);
}
exports.selectMergeMethod = selectMergeMethod;
function mergeMatched(matched, clashResolveOptions, joinMode) {
    const returnData = [];
    let resolveClash = clashResolveOptions.resolveClash;
    const mergeIntoSingleObject = selectMergeMethod(clashResolveOptions);
    for (const match of matched) {
        let { entry, matches } = match;
        let json = {};
        let binary = {};
        if (resolveClash === 'addSuffix') {
            let suffix1 = '1';
            let suffix2 = '2';
            if (joinMode === 'enrichInput2') {
                [suffix1, suffix2] = [suffix2, suffix1];
            }
            [entry] = addSuffixToEntriesKeys([entry], suffix1);
            matches = addSuffixToEntriesKeys(matches, suffix2);
            json = mergeIntoSingleObject({ ...entry.json }, ...matches.map((item) => item.json));
            binary = mergeIntoSingleObject({ ...entry.binary }, ...matches.map((item) => item.binary));
        }
        else {
            let preferInput1 = 'preferInput1';
            let preferInput2 = 'preferInput2';
            if (joinMode === 'enrichInput2') {
                [preferInput1, preferInput2] = [preferInput2, preferInput1];
            }
            if (resolveClash === undefined) {
                resolveClash = 'preferInput2';
            }
            if (resolveClash === preferInput1) {
                const [firstMatch, ...restMatches] = matches;
                json = mergeIntoSingleObject({ ...firstMatch.json }, ...restMatches.map((item) => item.json), entry.json);
                binary = mergeIntoSingleObject({ ...firstMatch.binary }, ...restMatches.map((item) => item.binary), entry.binary);
            }
            if (resolveClash === preferInput2) {
                json = mergeIntoSingleObject({ ...entry.json }, ...matches.map((item) => item.json));
                binary = mergeIntoSingleObject({ ...entry.binary }, ...matches.map((item) => item.binary));
            }
        }
        const pairedItem = [
            entry.pairedItem,
            ...matches.map((m) => m.pairedItem),
        ];
        returnData.push({
            json,
            binary,
            pairedItem,
        });
    }
    return returnData;
}
exports.mergeMatched = mergeMatched;
function checkMatchFieldsInput(data) {
    if (data.length === 1 && data[0].field1 === '' && data[0].field2 === '') {
        throw new Error('You need to define at least one pair of fields in "Fields to Match" to match on');
    }
    for (const [index, pair] of data.entries()) {
        if (pair.field1 === '' || pair.field2 === '') {
            throw new Error(`You need to define both fields in "Fields to Match" for pair ${index + 1},
				 field 1 = '${pair.field1}'
				 field 2 = '${pair.field2}'`);
        }
    }
    return data;
}
exports.checkMatchFieldsInput = checkMatchFieldsInput;
function checkInput(input, fields, disableDotNotation, inputLabel) {
    for (const field of fields) {
        const isPresent = (input || []).some((entry) => {
            if (disableDotNotation) {
                return entry.json.hasOwnProperty(field);
            }
            return (0, lodash_1.get)(entry.json, field, undefined) !== undefined;
        });
        if (!isPresent) {
            throw new Error(`Field '${field}' is not present in any of items in '${inputLabel}'`);
        }
    }
    return input;
}
exports.checkInput = checkInput;
function addSourceField(data, sourceField) {
    return data.map((entry) => {
        const json = {
            ...entry.json,
            _source: sourceField,
        };
        return {
            ...entry,
            json,
        };
    });
}
exports.addSourceField = addSourceField;
//# sourceMappingURL=GenericFunctions.js.map