"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Helpers = __importStar(require("../../../test/nodes/Helpers"));
const ExecuteWorkflow_1 = require("../../../test/nodes/ExecuteWorkflow");
const path_1 = __importDefault(require("path"));
describe('Test Move Binary Data Node', () => {
    beforeEach(async () => {
        await Helpers.initBinaryDataManager();
    });
    const workflow = Helpers.readJsonFileSync('nodes/MoveBinaryData/test/MoveBinaryData.workflow.json');
    const node = workflow.nodes.find((n) => n.name === 'Read Binary File');
    node.parameters.filePath = path_1.default.join(__dirname, 'data', 'sample.json');
    const tests = [
        {
            description: 'nodes/MoveBinaryData/test/MoveBinaryData.workflow.json',
            input: {
                workflowData: workflow,
            },
            output: {
                nodeData: {
                    'Binary to JSON': [
                        [
                            {
                                json: {
                                    id: 1,
                                    title: 'My Title',
                                    description: 'Lorem Ipsum ...',
                                },
                            },
                        ],
                    ],
                    'Binary to JSON - No Set All Data': [
                        [
                            {
                                json: {
                                    data: '{\n\t"id": 1,\n\t"title": "My Title",\n\t"description": "Lorem Ipsum ..."\n}',
                                },
                            },
                        ],
                    ],
                    'Binary to JSON - No Set All Data + Json Parse': [
                        [
                            {
                                json: {
                                    data: {
                                        id: 1,
                                        title: 'My Title',
                                        description: 'Lorem Ipsum ...',
                                    },
                                },
                            },
                        ],
                    ],
                    'Binary to JSON - No Set All Data + Encoding base64': [
                        [
                            {
                                json: {
                                    data: 'ewoJImlkIjogMSwKCSJ0aXRsZSI6ICJNeSBUaXRsZSIsCgkiZGVzY3JpcHRpb24iOiAiTG9yZW0gSXBzdW0gLi4uIgp9',
                                },
                            },
                        ],
                    ],
                    'Binary to JSON - Keep as Base64': [
                        [
                            {
                                json: {
                                    data: 'ewoJImlkIjogMSwKCSJ0aXRsZSI6ICJNeSBUaXRsZSIsCgkiZGVzY3JpcHRpb24iOiAiTG9yZW0gSXBzdW0gLi4uIgp9',
                                },
                            },
                        ],
                    ],
                    'JSON to Binary': [
                        [
                            {
                                json: {},
                                binary: {
                                    data: {
                                        data: 'eyJpZCI6MSwidGl0bGUiOiJNeSBUaXRsZSIsImRlc2NyaXB0aW9uIjoiTG9yZW0gSXBzdW0gLi4uIn0=',
                                        mimeType: 'application/json',
                                        fileType: 'json',
                                        fileSize: '59 B',
                                    },
                                },
                            },
                        ],
                    ],
                    'JSON to Binary - No Convert All Data': [
                        [
                            {
                                json: {
                                    id: 1,
                                    description: 'Lorem Ipsum ...',
                                },
                                binary: {
                                    data: {
                                        data: 'Ik15IFRpdGxlIg==',
                                        mimeType: 'application/json',
                                        fileType: 'json',
                                        fileSize: '10 B',
                                        fileName: 'example.json',
                                    },
                                },
                            },
                        ],
                    ],
                },
            },
        },
    ];
    const nodeTypes = Helpers.setup(tests);
    for (const testData of tests) {
        test(testData.description, async () => {
            const { result } = await (0, ExecuteWorkflow_1.executeWorkflow)(testData, nodeTypes);
            const resultNodeData = Helpers.getResultNodeData(result, testData);
            resultNodeData.forEach(({ nodeName, resultData }) => expect(resultData).toEqual(testData.output.nodeData[nodeName]));
            expect(result.finished).toEqual(true);
        });
    }
});
//# sourceMappingURL=MoveBinaryData.test.js.map