"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CredentialTypes = void 0;
const n8n_core_1 = require("n8n-core");
const constants_1 = require("./constants");
class CredentialTypesClass {
    constructor(nodesAndCredentials) {
        this.nodesAndCredentials = nodesAndCredentials;
        nodesAndCredentials.credentialTypes = this;
    }
    recognizes(type) {
        return type in this.knownCredentials || type in this.loadedCredentials;
    }
    getByName(credentialType) {
        return this.getCredential(credentialType).type;
    }
    getNodeTypesToTestWith(type) {
        var _a, _b;
        return (_b = (_a = this.knownCredentials[type]) === null || _a === void 0 ? void 0 : _a.nodesToTestWith) !== null && _b !== void 0 ? _b : [];
    }
    getParentTypes(typeName) {
        const credentialType = this.getByName(typeName);
        if ((credentialType === null || credentialType === void 0 ? void 0 : credentialType.extends) === undefined)
            return [];
        const types = [];
        credentialType.extends.forEach((type) => {
            types.push(type);
            types.push(...this.getParentTypes(type));
        });
        return types;
    }
    getCredential(type) {
        const loadedCredentials = this.loadedCredentials;
        if (type in loadedCredentials) {
            return loadedCredentials[type];
        }
        const knownCredentials = this.knownCredentials;
        if (type in knownCredentials) {
            const { className, sourcePath } = knownCredentials[type];
            const loaded = (0, n8n_core_1.loadClassInIsolation)(sourcePath, className);
            loadedCredentials[type] = { sourcePath, type: loaded };
            return loadedCredentials[type];
        }
        throw new Error(`${constants_1.RESPONSE_ERROR_MESSAGES.NO_CREDENTIAL}: ${type}`);
    }
    get loadedCredentials() {
        return this.nodesAndCredentials.loaded.credentials;
    }
    get knownCredentials() {
        return this.nodesAndCredentials.known.credentials;
    }
}
let credentialTypesInstance;
function CredentialTypes(nodesAndCredentials) {
    if (!credentialTypesInstance) {
        if (nodesAndCredentials) {
            credentialTypesInstance = new CredentialTypesClass(nodesAndCredentials);
        }
        else {
            throw new Error('CredentialTypes not initialized yet');
        }
    }
    return credentialTypesInstance;
}
exports.CredentialTypes = CredentialTypes;
//# sourceMappingURL=CredentialTypes.js.map