"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeTypes = exports.NodeTypesClass = void 0;
const n8n_core_1 = require("n8n-core");
const n8n_workflow_1 = require("n8n-workflow");
const constants_1 = require("./constants");
class NodeTypesClass {
    constructor(nodesAndCredentials) {
        this.nodesAndCredentials = nodesAndCredentials;
        this.applySpecialNodeParameters();
    }
    getWithSourcePath(nodeTypeName, version) {
        const nodeType = this.getNode(nodeTypeName);
        if (!nodeType) {
            throw new Error(`Unknown node type: ${nodeTypeName}`);
        }
        const { description } = n8n_workflow_1.NodeHelpers.getVersionedNodeType(nodeType.type, version);
        return { description: { ...description }, sourcePath: nodeType.sourcePath };
    }
    getByName(nodeType) {
        return this.getNode(nodeType).type;
    }
    getByNameAndVersion(nodeType, version) {
        return n8n_workflow_1.NodeHelpers.getVersionedNodeType(this.getNode(nodeType).type, version);
    }
    applySpecialNodeParameters() {
        for (const nodeTypeData of Object.values(this.loadedNodes)) {
            const nodeType = n8n_workflow_1.NodeHelpers.getVersionedNodeType(nodeTypeData.type);
            n8n_workflow_1.NodeHelpers.applySpecialNodeParameters(nodeType);
        }
    }
    getNode(type) {
        const loadedNodes = this.loadedNodes;
        if (type in loadedNodes) {
            return loadedNodes[type];
        }
        const knownNodes = this.knownNodes;
        if (type in knownNodes) {
            const { className, sourcePath } = knownNodes[type];
            const loaded = (0, n8n_core_1.loadClassInIsolation)(sourcePath, className);
            n8n_workflow_1.NodeHelpers.applySpecialNodeParameters(loaded);
            loadedNodes[type] = { sourcePath, type: loaded };
            return loadedNodes[type];
        }
        throw new Error(`${constants_1.RESPONSE_ERROR_MESSAGES.NO_NODE}: ${type}`);
    }
    get loadedNodes() {
        return this.nodesAndCredentials.loaded.nodes;
    }
    get knownNodes() {
        return this.nodesAndCredentials.known.nodes;
    }
}
exports.NodeTypesClass = NodeTypesClass;
let nodeTypesInstance;
function NodeTypes(nodesAndCredentials) {
    if (!nodeTypesInstance) {
        if (nodesAndCredentials) {
            nodeTypesInstance = new NodeTypesClass(nodesAndCredentials);
        }
        else {
            throw new Error('NodeTypes not initialized yet');
        }
    }
    return nodeTypesInstance;
}
exports.NodeTypes = NodeTypes;
//# sourceMappingURL=NodeTypes.js.map