"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const typeorm_1 = require("typeorm");
const ActiveWorkflowRunner = __importStar(require("../../../../ActiveWorkflowRunner"));
const config_1 = __importDefault(require("../../../../config"));
const WorkflowEntity_1 = require("../../../../databases/entities/WorkflowEntity");
const InternalHooksManager_1 = require("../../../../InternalHooksManager");
const ExternalHooks_1 = require("../../../../ExternalHooks");
const WorkflowHelpers_1 = require("../../../../WorkflowHelpers");
const global_middleware_1 = require("../../shared/middlewares/global.middleware");
const pagination_service_1 = require("../../shared/services/pagination.service");
const users_service_1 = require("../users/users.service");
const workflows_service_1 = require("./workflows.service");
const workflows_services_1 = require("../../../../workflows/workflows.services");
module.exports = {
    createWorkflow: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        async (req, res) => {
            const workflow = req.body;
            workflow.active = false;
            if (!(0, workflows_service_1.hasStartNode)(workflow)) {
                workflow.nodes.push((0, workflows_service_1.getStartNode)());
            }
            await (0, WorkflowHelpers_1.replaceInvalidCredentials)(workflow);
            (0, WorkflowHelpers_1.addNodeIds)(workflow);
            const role = await (0, users_service_1.getWorkflowOwnerRole)();
            const createdWorkflow = await (0, workflows_service_1.createWorkflow)(workflow, req.user, role);
            await (0, ExternalHooks_1.ExternalHooks)().run('workflow.afterCreate', [createdWorkflow]);
            void InternalHooksManager_1.InternalHooksManager.getInstance().onWorkflowCreated(req.user, createdWorkflow, true);
            return res.json(createdWorkflow);
        },
    ],
    deleteWorkflow: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        async (req, res) => {
            const { id: workflowId } = req.params;
            const workflow = await workflows_services_1.WorkflowsService.delete(req.user, workflowId);
            if (!workflow) {
                return res.status(404).json({ message: 'Not Found' });
            }
            return res.json(workflow);
        },
    ],
    getWorkflow: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        async (req, res) => {
            const { id } = req.params;
            const sharedWorkflow = await (0, workflows_service_1.getSharedWorkflow)(req.user, id);
            if (!sharedWorkflow) {
                return res.status(404).json({ message: 'Not Found' });
            }
            void InternalHooksManager_1.InternalHooksManager.getInstance().onUserRetrievedWorkflow({
                user_id: req.user.id,
                public_api: true,
            });
            return res.json(sharedWorkflow.workflow);
        },
    ],
    getWorkflows: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        global_middleware_1.validCursor,
        async (req, res) => {
            const { offset = 0, limit = 100, active = undefined, tags = undefined } = req.query;
            let workflows;
            let count;
            const where = {
                ...(active !== undefined && { active }),
            };
            const query = {
                skip: offset,
                take: limit,
                where,
                ...(!config_1.default.getEnv('workflowTagsDisabled') && { relations: ['tags'] }),
            };
            if ((0, users_service_1.isInstanceOwner)(req.user)) {
                if (tags) {
                    const workflowIds = await (0, workflows_service_1.getWorkflowIdsViaTags)((0, workflows_service_1.parseTagNames)(tags));
                    Object.assign(where, { id: (0, typeorm_1.In)(workflowIds) });
                }
                workflows = await (0, workflows_service_1.getWorkflows)(query);
                count = await (0, workflows_service_1.getWorkflowsCount)(query);
            }
            else {
                const options = {};
                if (tags) {
                    options.workflowIds = await (0, workflows_service_1.getWorkflowIdsViaTags)((0, workflows_service_1.parseTagNames)(tags));
                }
                const sharedWorkflows = await (0, workflows_service_1.getSharedWorkflows)(req.user, options);
                if (!sharedWorkflows.length) {
                    return res.status(200).json({
                        data: [],
                        nextCursor: null,
                    });
                }
                const workflowsIds = sharedWorkflows.map((shareWorkflow) => shareWorkflow.workflowId);
                Object.assign(where, { id: (0, typeorm_1.In)(workflowsIds) });
                workflows = await (0, workflows_service_1.getWorkflows)(query);
                count = await (0, workflows_service_1.getWorkflowsCount)(query);
            }
            void InternalHooksManager_1.InternalHooksManager.getInstance().onUserRetrievedAllWorkflows({
                user_id: req.user.id,
                public_api: true,
            });
            return res.json({
                data: workflows,
                nextCursor: (0, pagination_service_1.encodeNextCursor)({
                    offset,
                    limit,
                    numberOfTotalRecords: count,
                }),
            });
        },
    ],
    updateWorkflow: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        async (req, res) => {
            const { id } = req.params;
            const updateData = new WorkflowEntity_1.WorkflowEntity();
            Object.assign(updateData, req.body);
            const sharedWorkflow = await (0, workflows_service_1.getSharedWorkflow)(req.user, id);
            if (!sharedWorkflow) {
                return res.status(404).json({ message: 'Not Found' });
            }
            if (!(0, workflows_service_1.hasStartNode)(updateData)) {
                updateData.nodes.push((0, workflows_service_1.getStartNode)());
            }
            await (0, WorkflowHelpers_1.replaceInvalidCredentials)(updateData);
            (0, WorkflowHelpers_1.addNodeIds)(updateData);
            const workflowRunner = ActiveWorkflowRunner.getInstance();
            if (sharedWorkflow.workflow.active) {
                await workflowRunner.remove(id);
            }
            try {
                await (0, workflows_service_1.updateWorkflow)(sharedWorkflow.workflowId, updateData);
            }
            catch (error) {
                if (error instanceof Error) {
                    return res.status(400).json({ message: error.message });
                }
            }
            if (sharedWorkflow.workflow.active) {
                try {
                    await workflowRunner.add(sharedWorkflow.workflowId, 'update');
                }
                catch (error) {
                    if (error instanceof Error) {
                        return res.status(400).json({ message: error.message });
                    }
                }
            }
            const updatedWorkflow = await (0, workflows_service_1.getWorkflowById)(sharedWorkflow.workflowId);
            await (0, ExternalHooks_1.ExternalHooks)().run('workflow.afterUpdate', [updateData]);
            void InternalHooksManager_1.InternalHooksManager.getInstance().onWorkflowSaved(req.user, updateData, true);
            return res.json(updatedWorkflow);
        },
    ],
    activateWorkflow: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        async (req, res) => {
            const { id } = req.params;
            const sharedWorkflow = await (0, workflows_service_1.getSharedWorkflow)(req.user, id);
            if (!sharedWorkflow) {
                return res.status(404).json({ message: 'Not Found' });
            }
            if (!sharedWorkflow.workflow.active) {
                try {
                    await ActiveWorkflowRunner.getInstance().add(sharedWorkflow.workflowId, 'activate');
                }
                catch (error) {
                    if (error instanceof Error) {
                        return res.status(400).json({ message: error.message });
                    }
                }
                await (0, workflows_service_1.setWorkflowAsActive)(sharedWorkflow.workflow);
                sharedWorkflow.workflow.active = true;
                return res.json(sharedWorkflow.workflow);
            }
            return res.json(sharedWorkflow.workflow);
        },
    ],
    deactivateWorkflow: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        async (req, res) => {
            const { id } = req.params;
            const sharedWorkflow = await (0, workflows_service_1.getSharedWorkflow)(req.user, id);
            if (!sharedWorkflow) {
                return res.status(404).json({ message: 'Not Found' });
            }
            const workflowRunner = ActiveWorkflowRunner.getInstance();
            if (sharedWorkflow.workflow.active) {
                await workflowRunner.remove(sharedWorkflow.workflowId);
                await (0, workflows_service_1.setWorkflowAsInactive)(sharedWorkflow.workflow);
                sharedWorkflow.workflow.active = false;
                return res.json(sharedWorkflow.workflow);
            }
            return res.json(sharedWorkflow.workflow);
        },
    ],
};
//# sourceMappingURL=workflows.handler.js.map