"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupPushHandler = exports.setupPushServer = exports.getPushInstance = void 0;
const http_1 = require("http");
const ws_1 = require("ws");
const url_1 = require("url");
const config_1 = __importDefault(require("../config"));
const jwt_1 = require("../auth/jwt");
const constants_1 = require("../constants");
const sse_push_1 = require("./sse.push");
const websocket_push_1 = require("./websocket.push");
const useWebSockets = config_1.default.getEnv('push.backend') === 'websocket';
let pushInstance;
const getPushInstance = () => {
    if (!pushInstance)
        pushInstance = useWebSockets ? new websocket_push_1.WebSocketPush() : new sse_push_1.SSEPush();
    return pushInstance;
};
exports.getPushInstance = getPushInstance;
const setupPushServer = (restEndpoint, server, app) => {
    if (useWebSockets) {
        const wsServer = new ws_1.Server({ noServer: true });
        server.on('upgrade', (request, socket, head) => {
            if ((0, url_1.parse)(request.url).pathname === `/${restEndpoint}/push`) {
                wsServer.handleUpgrade(request, socket, head, (ws) => {
                    request.ws = ws;
                    const response = new http_1.ServerResponse(request);
                    response.writeHead = (statusCode) => {
                        if (statusCode > 200)
                            ws.close();
                        return response;
                    };
                    app.handle(request, response);
                });
            }
        });
    }
};
exports.setupPushServer = setupPushServer;
const setupPushHandler = (restEndpoint, app, isUserManagementEnabled) => {
    const push = (0, exports.getPushInstance)();
    const endpoint = `/${restEndpoint}/push`;
    const pushValidationMiddleware = async (req, res, next) => {
        var _a, _b;
        const ws = req.ws;
        const { sessionId } = req.query;
        if (sessionId === undefined) {
            if (ws) {
                ws.send('The query parameter "sessionId" is missing!');
                ws.close(400);
            }
            else {
                next(new Error('The query parameter "sessionId" is missing!'));
            }
            return;
        }
        if (isUserManagementEnabled) {
            try {
                const authCookie = (_b = (_a = req.cookies) === null || _a === void 0 ? void 0 : _a[constants_1.AUTH_COOKIE_NAME]) !== null && _b !== void 0 ? _b : '';
                await (0, jwt_1.resolveJwt)(authCookie);
            }
            catch (error) {
                if (ws) {
                    ws.send(`Unauthorized: ${error.message}`);
                    ws.close(401);
                }
                else {
                    res.status(401).send('Unauthorized');
                }
                return;
            }
        }
        next();
    };
    app.use(endpoint, pushValidationMiddleware, (req, res) => {
        if (req.ws) {
            push.add(req.query.sessionId, req.ws);
        }
        else if (!useWebSockets) {
            push.add(req.query.sessionId, { req, res });
        }
        else {
            res.status(401).send('Unauthorized');
        }
    });
};
exports.setupPushHandler = setupPushHandler;
//# sourceMappingURL=index.js.map